package sun.net.httpserver;

import java.io.IOException;
import java.io.InputStream;

class ChunkedInputStream extends LeftOverInputStream
{
    private int remaining;
    private boolean needToReadHeader;
    static final char CR = '\r';
    static final char LF = '\n';
    private static final int MAX_CHUNK_HEADER_SIZE = 2050;
    
    ChunkedInputStream(ExchangeImpl exchangeImpl, InputStream inputStream) {
        super(exchangeImpl, inputStream);
        this.needToReadHeader = true;
    }
    
    private int numeric(char[] array, int n) throws IOException {
        assert array.length >= n;
        int n2 = 0;
        int n4;
        for (int i = 0; i < n; n2 = n2 * 16 + n4, ++i) {
            int n3 = array[i];
            if (n3 >= '0' && n3 <= '9') {
                n4 = n3 - 48;
            }
            else if (n3 >= 'a' && n3 <= 'f') {
                n4 = n3 - 97 + 10;
            }
            else if (n3 >= 'A' && n3 <= 'F') {
                n4 = n3 - 65 + 10;
            }
            else {
                throw new IOException("invalid chunk length");
            }
        }
        return n2;
    }
    
    private int readChunkHeader() throws IOException {
        int n = 0;
        char[] array = new char[16];
        int n2 = 0;
        int n3 = 0;
        int n4 = 0;
        int read;
        while ((read = this.in.read()) != -1) {
            int n5 = (char)read;
            ++n4;
            if (n2 == array.length - 1 || n4 > 2050) {
                throw new IOException("invalid chunk header");
            }
            if (n != 0) {
                if (n5 == '\n') {
                    return this.numeric(array, n2);
                }
                n = 0;
                if (n3 == 0) {
                    array[n2++] = n5;
                }
                else {
                    continue;
                }
            }
            else if (n5 == '\r') {
                n = 1;
            }
            else if (n5 == ';') {
                n3 = 1;
            }
            else if (n3 == 0) {
                array[n2++] = n5;
            }
            else {
                continue;
            }
        }
        throw new IOException("end of stream reading chunk header");
    }
    
    @Override
    protected int readImpl(byte[] b, int off, int remaining) throws IOException {
        if (this.eof) {
            return -1;
        }
        if (this.needToReadHeader) {
            this.remaining = this.readChunkHeader();
            if (this.remaining == 0) {
                this.eof = true;
                this.consumeCRLF();
                this.t.getServerImpl().requestCompleted(this.t.getConnection());
                return -1;
            }
            this.needToReadHeader = false;
        }
        if (remaining > this.remaining) {
            remaining = this.remaining;
        }
        int read = this.in.read(b, off, remaining);
        if (read > -1) {
            this.remaining -= read;
        }
        if (this.remaining == 0) {
            this.needToReadHeader = true;
            this.consumeCRLF();
        }
        return read;
    }
    
    private void consumeCRLF() throws IOException {
        if ((char)this.in.read() != '\r') {
            throw new IOException("invalid chunk end");
        }
        if ((char)this.in.read() != '\n') {
            throw new IOException("invalid chunk end");
        }
        return;
    }
    
    @Override
    public int available() throws IOException {
        if (this.eof || this.closed) {
            return 0;
        }
        int available = this.in.available();
        return (available > this.remaining) ? this.remaining : available;
    }
    
    @Override
    public boolean isDataBuffered() throws IOException {
        assert !(!this.eof);
        return this.in.available() > 0;
    }
    
    @Override
    public boolean markSupported() {
        return false;
    }
    
    @Override
    public void mark(int n) {
    }
    
    @Override
    public void reset() throws IOException {
        throw new IOException("mark/reset not supported");
    }
}
